/*
 * Copyright 2021-2023 VMware, Inc.
 * SPDX-License-Identifier: Apache-2.0
 */

.jp-vdk-input-wrapper {
  display: flex;
  flex-direction: column;
  margin-top: 15px;
}

.hidden {
  display: none;
}

.jp-vdk-checkbox-wrappers {
  display: flex;
  justify-content: flex-start;
  align-items: center;
  margin-left: 0;
  padding-left: 0;
}

.jp-vdk-checkbox {
  margin-right: 10px;
}

#enable {
  margin-top: 10px;
  margin-left: 0;
}

.vdk-run-error-message {
  color: #c0392b;
  padding: 16px;
  font-weight: bold;
  display: flex;
  align-items: center;
}

.vdk-run-error-message p {
  padding-bottom: 10px;
}

.vdk-run-error-message a {
  color: #11239a;
  text-decoration: underline;
}

.vdk-run-dialog-message {
  font-size: 16px;
  color: #33b53a;
}
.vdk-run-dialog-message-container {
  display: flex;
  align-items: center;
}

.vdk-status-dialog-message {
  font-size: 16px;
  padding: 20px;
}

.jp-vdk-check-status-button {
  position: absolute;
  right: 1px;
  border-radius: 5px;
  border: none;
}

.jp-vdk-status-button-content {
  display: flex;
  align-items: center;
}

.jp-vdk-check-status-button .jp-vdk-logo {
  position: static;
  padding-right: 5px;
}

.jp-vdk-check-status-button:hover {
  background-color: #e0e0e0;
  cursor: pointer;
}

.vdk-dialog-check-icon {
  margin-right: 10px;
}

.vdk-convert-to-notebook-error-message {
  display: flex;
  font-size: 16px;
  color: #c0392b;
}

.vdk-dialog-error-icon .jp-icon3[fill] {
  fill: #c0392b;
}

.vdk-dialog-check-icon .jp-icon3[fill] {
  fill: #33b53a;
}

.vdk-error-dialog {
  font-size: 16px;
  color: #c0392b;
  display: flex;
  align-items: center;
}

.vdk-dialog-error-icon {
  margin-right: 10px;
  display: flex;
  align-items: center;
}

.jp-vdk-login2-input {
  width: 80%;
}

.jp-vdk-login2-button {
  padding: 8px 16px;
  border: 1px solid #ccc;
  background: linear-gradient(to bottom, #f7f7f7, #e7e7e7);
  font-weight: 600;
  border-radius: 2px;
  cursor: pointer;
  color: #333;
  text-align: center;
  font-size: 14px;
  transition: background 0.3s;
}

.jp-vdk-login2-button:hover {
  background: linear-gradient(to bottom, #e7e7e7, #d7d7d7);
}

.jp-vdk-login2-dialog-container {
  text-align: left;
  padding: 20px;
  width: 600px;
}
