/*
 * Copyright 2023-2023 VMware, Inc.
 * SPDX-License-Identifier: Apache-2.0
 */

export enum VdkOption {
  NAME = 'jobName',
  TEAM = 'jobTeam',
  PATH = 'jobPath',
  ARGUMENTS = 'jobArguments',
  DEPLOYMENT_REASON = 'deploymentReason'
}
